#include <cstdlib>
#include <iostream>

#include "pad.h"
#include "picBox.h"

#ifndef NULL
#define NULL 0
#endif

using namespace std;

pad::pad(QWidget * parent, Qt::WindowFlags f):QWidget(parent, f)
{
	m_buffer = new QPixmap;
	m_background = new QPixmap;
	m_parent = NULL;
	setMouseTracking(true);
}

pad::pad(int x, int y)
{
	setMouseTracking(true);
	m_background = new QPixmap(x, y);
	m_buffer = new QPixmap(x, y);
	m_background->fill();
	earse();
	m_minWidth = x;
	m_minHeight = y;
	setMinimumSize(m_minWidth, m_minHeight);
	m_parent = NULL;
}

void pad::paintEvent(QPaintEvent *event)
{
	QPainter painter(this);
	painter.drawPixmap(0, 0, m_minWidth, m_minHeight, *m_buffer);
}

bool pad::loadImg(const QString &filename)
{
	QImage img;
	if (!img.load(filename))
	{
		return false;
	}
	
	m_minWidth = img.width();
	m_minHeight = img.height();
	
	delete m_background;
	delete m_buffer;
	
	m_background =  new QPixmap(QPixmap::fromImage(img));
	m_buffer = new QPixmap(m_minWidth, m_minHeight);
	*m_buffer = m_background->copy();
	
	setMinimumSize(m_minWidth, m_minHeight);
	repaint(0, 0, m_minWidth, m_minHeight);
	return true;
}

void pad::setBackground(QPixmap *background)
{
	delete m_background;
	delete m_buffer;
	
	m_background = new QPixmap(*background);
	m_buffer = new QPixmap(*background);
	m_minWidth = background->width();
	m_minHeight = background->height();
	setMinimumSize(m_minWidth, m_minHeight);
}

void pad::mousePressEvent(QMouseEvent *event)
{
	if (m_parent != NULL)
	{
		m_parent->mouseDownEvent(event);
	}
}

void pad::mouseMoveEvent(QMouseEvent *event)
{
	if (m_parent != NULL)
	{
		m_parent->mouseMoveEvent(event);
	}
}

void pad::mouseReleaseEvent(QMouseEvent *event)
{
	if (m_parent != NULL)
	{
		m_parent->mouseUpEvent(event);
	}
}

void pad::threshold(int v)
{
	int r, g, b;
	int x, y;
	int gray;
	QRgb c;
	QPainter p(m_buffer);
	QImage img(m_buffer->toImage());
	
	for (y=0; y<img.height(); ++y)
	{
		for (x=0; x<img.width(); ++x)
		{
			c = img.pixel(x, y);
			r = qRed(c);
			g = qGreen(c);
			b = qBlue(c);
			gray = (r + g + b) / 3;
			if (gray < v)
			{
				p.setPen(QColor::fromRgb(0, 0, 0));
			}
			else
			{
				p.setPen(QColor::fromRgb(255, 255, 255));
			}
			p.drawPoint(x, y);
		}
	}
	repaint(0, 0, m_minWidth, m_minHeight);
}

void pad::setPoint(int x, int y,int r, int g, int b)
{
	if (x>=0 && y>=0 && x<m_minWidth && y<m_minHeight)
	{
		QPainter painter(m_buffer);
		painter.setPen(QColor::fromRgb(r, g, b));
		painter.drawPoint(x, y);
		repaint(x-1, y-1, 3, 3);
	}
}

void pad::setParent(picBox *parent)
{
	m_parent = parent;
}

void pad::earse(void)
{
	*m_buffer = m_background->copy(0, 0, m_minWidth, m_minHeight);
	repaint(0, 0, m_minWidth, m_minHeight);
}

void pad::drawCircle(int x, int y, int ra, int r, int g, int b)
{
	QPainter p(m_buffer);
	QPen pen;
	pen.setWidth(1);
	pen.setStyle(Qt::SolidLine);
	pen.setColor(QColor::fromRgb(r, g, b));
	p.setPen(pen);
	p.drawEllipse(x-ra, y-ra, ra<<1, ra<<1);
	repaint(x-ra-1, y-ra-1, ra<<1+2, ra<<1+2);
}

QPixmap &pad::getBuffer(void)
{
	
}

void pad::buffer2background(void)
{
	*m_background = m_buffer->copy(0, 0, m_minWidth, m_minHeight);
}

