#include <cstdio>
#include <cmath>

#include "util.h"


bool gauss(double in[5][6], double out[5]);
bool gauss2(double in[2][3], double out[2]);

void calibrate(double cameraPoints[16][2], double worldPoints[16][2], double rotate[3][3], double translation[3], double &f)
{
	double A[5][6];
	double ideal[16][2];
	double deals[5][6];
	double s[5];
	double xc, yc;
	double B[5][3];
	double Bi[2][3];
	double ui[2];
	double u[2];
	int uCnt;
	
	int x, y;
	double sr;
	int t[5];
	int st[2];
	
	double allF;
	double allR[3][3];
	double allT[3];
	int ansCnt;
	
	allF = 0.0;
	for (y=0; y<3; ++y)
	{
		allT[y] = 0.0;
		for (x=0; x<3; ++x)
		{
			allR[y][x] = 0.0;
		}
	}
	ansCnt = 0;
	for (y=0; y<16; ++y)
	{
		ideal[y][0] = (cameraPoints[y][0] - 384.0) / 120.0;
		ideal[y][1] = (cameraPoints[y][1] - 288.0) / 120.0;
//		printf("ideal x: %.4lf,\ty: %.4lf\n", ideal[y][0], ideal[y][1]);
	}
	//printf("\n");
	for (t[0]=0; t[0]<12; ++t[0])
	{
		for (t[1]=t[0]+1; t[1]<13; ++t[1])
		{
			for (t[2]=t[1]+1; t[2]<14; ++t[2])
			{
				for (t[3]=t[2]+1; t[3]<15; ++t[3])
				{
					for (t[4]=t[3]+1; t[4]<16; ++t[4])
					{

						for (y=0; y<5; ++y)
						{
							A[y][0] = worldPoints[t[y]][0] * ideal[t[y]][1];
							A[y][1] = worldPoints[t[y]][1] * ideal[t[y]][1];
							A[y][2] = ideal[t[y]][1];
							A[y][3] = -worldPoints[t[y]][0] * ideal[t[y]][0];
							A[y][4] = -worldPoints[t[y]][1] * ideal[t[y]][0];
							A[y][5] = ideal[t[y]][0];
						}
						
						for (y=0; y<5; ++y)
						{
							for (x=0; x<6; ++x)
							{
								//printf("%.3lf\t", A[y][x]);
							}
							//printf("\n");
						}
						//printf("\n");
						
						gauss(A, s);
						sr = 0.0;
						//printf("\ns:\n");
						for (y=0; y<5; ++y)
						{
							//printf("%.5lf\n", s[y]);
							if (y != 2)
							{
								sr += (s[y] * s[y]);
							}
						}
						if (fabs(s[0]*s[4] - s[3]*s[1]) < 1e-4)
						{
							continue;
						}
						translation[1] = sqrt((sr - sqrt(sr*sr - 4*(s[0]*s[4]-s[3]*s[1])*(s[0]*s[4]-s[3]*s[1]))) / (2*(s[0]*s[4] - s[3]*s[1])*(s[0]*s[4] - s[3]*s[1])));
						//printf("Ty=%.5lf\n", translation[1]);
						rotate[0][0] = s[0] * translation[1];
						rotate[0][1] = s[1] * translation[1];
						rotate[1][0] = s[3] * translation[1];
						rotate[1][1] = s[4] * translation[1];
						
						translation[0] = s[2] * translation[1];
						//printf("tx=%.5lf\n", translation[0]);
						xc = rotate[0][0] * worldPoints[t[0]][0] + rotate[0][1]*worldPoints[t[0]][1] + translation[0];
						yc = rotate[1][0] * worldPoints[t[0]][0] + rotate[1][1]*worldPoints[t[0]][1] + translation[1];
						
						//printf("check ty:     x:%.4lf\ty:%.4lf\n", xc, yc);
						
						if (!(worldPoints[t[0]][0]*xc>0 && worldPoints[t[0]][1]*yc>0))
						{
							//printf("re cal the ty\n");
							translation[1] = -translation[1];
							//printf("Ty=%.5lf\n", translation[1]);
							rotate[0][0] = s[0] * translation[1];
							rotate[0][1] = s[1] * translation[1];
							rotate[1][0] = s[3] * translation[1];
							rotate[1][1] = s[4] * translation[1];
							
							translation[0] = s[2] * translation[1];
							//printf("tx=%.5lf\n", translation[0]);
						}
						rotate[0][2] = sqrt(1 - rotate[0][0]*rotate[0][0] - rotate[0][1]*rotate[0][1]);
						rotate[1][2] = sqrt(1 - rotate[1][0]*rotate[1][0] - rotate[1][1]*rotate[1][1]);
						rotate[2][0] = (1 - rotate[0][0]*rotate[0][0] - rotate[0][1]*rotate[1][0]) / rotate[0][2];
						rotate[2][1] = (1 - rotate[0][1]*rotate[1][0] - rotate[1][1]*rotate[1][1]) / rotate[1][2];
						rotate[2][2] = -sqrt(1 - rotate[0][2]*rotate[2][0] - rotate[1][2]*rotate[2][1]);
						if ((1 - rotate[0][2]*rotate[2][0] - rotate[1][2]*rotate[2][1]) < 0)
						{
							printf("r9:%.5lf\t", rotate[2][2]);
							break;
						}
						if (rotate[0][0]*rotate[1][0] + rotate[0][1]*rotate[1][1] > 0)
						{
							rotate[1][2] = -rotate[1][2];
						}
						//printf("rotate:\n");
						for (y=0; y<3; ++y)
						{
							for (x=0; x<3; ++x)
							{
								//printf("%.4lf\t", rotate[y][x]);
							}
							//printf("\n");
						}
						//printf("B\n");
						for (y=0; y<5; ++y)
						{
							B[y][0] = floor(rotate[1][0]*worldPoints[t[y]][0] + rotate[1][1]*worldPoints[t[y]][1] + translation[1]);
							B[y][1] = floor(ideal[t[y]][1]);
							B[y][2] = (rotate[2][0]*worldPoints[t[y]][0] + rotate[2][1]*worldPoints[t[y]][1]) * ideal[t[y]][1];
							//printf("%.4lf\t%.4lf\t%.4lf\n", B[y][0], B[y][1], B[y][2]);
						}
						uCnt = 0;
						u[0] = 0.0;
						u[1] = 0.0;
						
						
						/*
						B[0][0] = -20.7641;
						B[1][0] = -7.4782;
						B[2][0] = 7.9075;
						B[3][0] = 16.3802;
						B[4][0] = 12.9768;
						
						B[0][1] = 1.2917;
						B[1][1] = 0.4833;
						B[2][1] = -0.5417;
						B[3][1] = -1.1833;
						B[4][1] = -0.90;
						
						B[0][2] = -21.0834;
						B[1][2] = -0.5921;
						B[2][2] = -7.9030;
						B[3][2] = -25.8661;
						B[4][2] = -14.6904;
						
						for (y=0; y<5; ++y)
						{
							//printf("%.4lf\t%.4lf\t%.4lf\n", B[y][0], B[y][1], B[y][2]);
						}
						*/
						
						for (st[0]=0; st[0]<4; ++st[0])
						{
							for (st[1]=st[0]+1; st[1]<5; ++st[1])
							{
								for (x=0; x<3; ++x)
								{
									Bi[0][x] = B[st[0]][x];
									Bi[1][x] = B[st[1]][x];
								}
								if (gauss2(Bi, ui))
								{
									u[0] += ui[0];
									u[1] += ui[1];
									++uCnt;
								}
							}
						}
						
						if (0 == uCnt)
						{
							continue;
						}
						u[0] = u[0] / static_cast<double>(uCnt);
						u[1] = u[1] / static_cast<double>(uCnt);
						//printf("f:%.4lf\t\tTz:%.5lf\n", u[0], u[1]);
						if (u[0] < 0)
						{
							u[0] = -u[0];
							u[1] = -u[1];
							rotate[0][2] = -rotate[0][2];
							rotate[1][2] = -rotate[1][2];
							rotate[2][0] = -rotate[2][0];
							rotate[2][1] = -rotate[2][1];
						}
						f = u[0];
						translation[2] = u[1];
						printf("r9:%.5lf\t", rotate[2][2]);
						allF += f;
						for (y=0; y<3; ++y)
						{
							allT[y] += translation[y];
							for (x=0; x<3; ++x)
							{
								allR[y][x] += rotate[y][x];
							}
						}
						++ansCnt;
					}
				}
			}
		}
	}
	printf("\n%d solutions\n", ansCnt);
	f = allF / static_cast<double>(ansCnt);
	for(y=0; y<3; ++y)
	{
		translation[y] = allT[y] / static_cast<double>(ansCnt);
		for (x=0; x<3; ++x)
		{
			rotate[y][x] = allR[y][x] / static_cast<double>(ansCnt);
		}
	}
	
	for (y=0; y<3; ++y)
	{
		for (x=0; x<3; ++x)
		{
			printf("%.4lf\t", rotate[y][x]);
		}
		printf("\n");
	}
	printf("\n");
	for (y=0; y<3; ++y)
	{
		printf("%.5lf\n", translation[y]);
	}
	printf("f=%.5lf\n", f);
	
}

bool gauss2(double in[2][3], double out[2])
{
	double B[2][3];
	int x, y;
	double tmp;
	for (y=0; y<2; ++y)
	{
		for (x=0; x<3; ++x)
		{
			B[y][x] = in[y][x];
		}
	}
	
	if (fabs(B[0][0]) < fabs(B[1][0]))
	{
		for (x=0; x<3; ++x)
		{
			tmp = B[0][x];
			B[0][x] = B[1][x];
			B[1][x] = tmp;
		}
	}
	if (fabs(B[0][0]) < 1e-4)
	{
		return false;
	}
	tmp = B[1][0] / B[0][0];
	B[1][0] = 0.0;
	B[1][1] -= (tmp * B[0][1]);
	B[1][2] -= (tmp * B[0][2]);
	if (fabs(B[1][1]) < 1e-4)
	{
		return false;
	}
	tmp = B[0][1] / B[1][1];
	B[0][1] = 0.0;
	B[0][0] -= (tmp * B[1][0]);
	B[0][2] -= (tmp * B[1][2]);
	if (fabs(B[0][0]) < 1e-4)
	{
		return false;
	}
	out[0] = B[0][2] / B[0][0];
	out[1] = B[1][2] / B[1][1];
	return true;
}

bool gauss(double in[5][6], double out[5])
{
	printf("gauss\n");
	int x, y;
	int t;
	int maxx;
	double tmp;
	double A[5][6];
	int sr;
	for (y=0; y<5; ++y)
	{
		for (x=0; x<6; ++x)
		{
			A[y][x] = in[y][x];
		}
	}
	for (t=0; t<5; ++t)
	{
		for (y=0; y<5; ++y)
		{
			for (x=0; x<6; ++x)
			{
				printf("%.3lf\t", A[y][x]);
			}
			printf("\n");
		}
		
		//find the max mian elem
		maxx = t;
		for (y=t+1; y<5; ++y)
		{
			if (fabs(A[y][t]) > fabs(A[maxx][t]))
			{
				maxx = y;
			}
		}
		printf("%.5lf\n", A[maxx][t]);
		if (fabs(A[maxx][t]) < 1e-5)
		{
			return false;
		}
		
		//change the row
		for (x=t; x<6; ++x)
		{
			tmp = A[maxx][x];
			A[maxx][x] = A[t][x];
			A[t][x] = tmp;
		}
		
		//down
		for (y=t+1; y<5; ++y)
		{
			tmp = A[y][t] / A[t][t];
			A[y][t] = 0;
			for (x=t+1; x<6; ++x)
			{
				A[y][x] -= A[t][x]*tmp;
			}
		}
	}
	for (t=4; t>=0; --t)
	{
		if (fabs(A[t][t]) < 1e-4)
		{
			return false;
		}
		for (y=t-1; y>=0; --y)
		{
			tmp = A[y][t] / A[t][t];
			A[y][t] = 0;
			A[y][5] -= A[t][5]*tmp;
		}
	}
	
	for (y=0; y<5; ++y)
	{
		for (x=0; x<6; ++x)
		{
			printf("%.3lf\t", A[y][x]);
		}
		printf("\n");
	}
	
	for (t=0; t<5; ++t)
	{
		if (fabs(A[t][t]) < 1e-4)
		{
			return false;
		}
		out[t] = A[t][5] / A[t][t];
	}
	return true;
}
