#include <cstdio>
#include <iostream>

#include <QtGui>

#include "window.h"
#include "picBox.h"
#include "util.h"

#define pi 3.14159265897932384626433832795

using namespace std;

window::window(QWidget *parent, Qt::WindowFlags flags):QMainWindow(parent, flags)
{
	m_cWidget = new QWidget;
	m_layout = new QHBoxLayout;
	m_frame = new QFrame;
	m_panelLayout = new QVBoxLayout;
	m_camera = new picBox;
	m_world = new picBox(100, 100);
	m_infoFrame = new QFrame;
	m_infoLayout = new QVBoxLayout;
	m_loadPicBtn = new QPushButton(tr("load pic"));
	m_thresholdBtn = new QPushButton(tr("threshold"));
	m_calibrationBtn = new QPushButton(tr("Calibration"));
	m_pointLbl = new QLabel(tr("point info\nworld(x, y)->image(x, y)\n"));
	m_closeBtn = new QPushButton(tr("close"));
	
	m_frame->setFrameShape(QFrame::Box);
	m_frame->setFrameShadow(QFrame::Raised);
	m_infoFrame->setFrameShape(QFrame::Box);
	m_infoFrame->setFrameShadow(QFrame::Raised);
	
	setCentralWidget(m_cWidget);
	m_cWidget->setLayout(m_layout);
	m_layout->addWidget(m_camera);
	m_layout->addWidget(m_frame);
	m_frame->setLayout(m_panelLayout);
	m_panelLayout->addWidget(m_world);
	m_panelLayout->addWidget(m_infoFrame);
	m_infoFrame->setLayout(m_infoLayout);
	m_infoLayout->addWidget(m_loadPicBtn);
	m_infoLayout->addWidget(m_thresholdBtn);
	m_infoLayout->addWidget(m_calibrationBtn);
	m_infoLayout->addWidget(m_pointLbl);
	m_infoLayout->addWidget(m_closeBtn);
	
	worldBg();
	const double sq2 = sqrt(2) / 2;
	m_worldPoints[0][0] = 45.0;
	m_worldPoints[0][1] = 0.0;
	m_worldPoints[1][0] = 45.0 * sq2;
	m_worldPoints[1][1] = 45.0 * sq2;
	m_worldPoints[2][0] = 0.0;
	m_worldPoints[2][1] = 45.0;
	m_worldPoints[3][0] = -45.0 * sq2;
	m_worldPoints[3][1] = 32.0;
	m_worldPoints[4][0] = -45.0;
	m_worldPoints[4][1] = 0.0;
	m_worldPoints[5][0] = -45.0 * sq2;
	m_worldPoints[5][1] = -45.0 * sq2;
	m_worldPoints[6][0] = 0.0;
	m_worldPoints[6][1] = -45.0;
	m_worldPoints[7][0] = 45.0 * sq2;
	m_worldPoints[7][1] = -45.0 * sq2;
	m_worldPoints[8][0] = 30.0;
	m_worldPoints[8][1] = 0.0;
	m_worldPoints[9][0] = 30.0 * sq2;
	m_worldPoints[9][1] = 30.0 * sq2;
	m_worldPoints[10][0] = 0.0;
	m_worldPoints[10][1] = 30.0;
	m_worldPoints[11][0] = -30.0 * sq2;
	m_worldPoints[11][1] = 30.0 * sq2;
	m_worldPoints[12][0] = -30.0;
	m_worldPoints[12][1] = 0.0;
	m_worldPoints[13][0] = -30.0 * sq2;
	m_worldPoints[13][1] = -30.0 * sq2;
	m_worldPoints[14][0] = 0.0;
	m_worldPoints[14][1] = -30.0;
	m_worldPoints[15][0] = 30.0 * sq2;
	m_worldPoints[15][1] = -30.0 * sq2;

	connect(m_loadPicBtn, SIGNAL(clicked()), this, SLOT(loadPic()));
	connect(m_thresholdBtn, SIGNAL(clicked()), this, SLOT(thresholdPic()));
	connect(m_calibrationBtn, SIGNAL(clicked()), this, SLOT(calibration()));
	connect(m_closeBtn, SIGNAL(clicked()), this, SLOT(close()));
	connect(m_camera, SIGNAL(mouseDown(QMouseEvent &)), this, SLOT(setCameraPoint(QMouseEvent &)));
	connect(m_world, SIGNAL(mouseDown(QMouseEvent &)), this, SLOT(worldMouseDown(QMouseEvent &)));
	connect(m_world, SIGNAL(mouseUp(QMouseEvent &)), this, SLOT(worldMouseUp(QMouseEvent &)));
	connect(m_world, SIGNAL(mouseMove(QMouseEvent &)), this, SLOT(worldMouseMove(QMouseEvent &)));
	m_worldMouseDown = false;
}



void window::loadPic(void)
{
	QString filename = QFileDialog::getOpenFileName(this, tr("Please choose a image file"), tr("."), tr("Images (*.jpg *.bmp *.png)"));
	m_camera->loadImg(filename);
}

void window::thresholdPic(void)
{
	m_camera->threshold(172);
}

void window::calibration(void)
{
	char buf[1000];
	QString str;
	m_stepIndex = 0;
	m_world->earse();
	m_camera->earse();
	m_world->drawCircle(static_cast<int>(m_worldPoints[0][0]+50.5), 50-static_cast<int>(m_worldPoints[0][1]+0.5), 3, 255, 0, 0);
	m_world->drawCircle(static_cast<int>(m_worldPoints[0][0]+50.5), 50-static_cast<int>(m_worldPoints[0][1]+0.5), 2, 255, 0, 0);
	m_pointLbl->setText(tr("point info\nworld(x, y)->image(x, y)\n"));
	str  = m_pointLbl->text();
	sprintf(buf, "%c: (%.1lf, %.1lf) -> ", 'A'+m_stepIndex, m_worldPoints[0][0], m_worldPoints[0][1]);
	str += buf;
	m_pointLbl->setText(str);
	
}


void window::setCameraPoint(QMouseEvent &event)
{
	
	char buf[1000];
	QString str;
	int i;
	if (m_stepIndex < 16)
	{
		//m_stepIndex = 16;
		m_cameraPoints[m_stepIndex][0] = event.x();
		m_cameraPoints[m_stepIndex][1] = event.y();
		str  = m_pointLbl->text();
		sprintf(buf, "(%d, %d)\n", static_cast<int>(m_cameraPoints[m_stepIndex][0]), static_cast<int>(m_cameraPoints[m_stepIndex][1]));
		str += buf;
		
		for (i=-3; i<=3; ++i)
		{
			m_camera->setPoint(event.x()+i, event.y()+i, 255, 0, 0);
			m_camera->setPoint(event.x()+i, event.y()-i, 255, 0, 0);
		}
		
		++m_stepIndex;
		if (m_stepIndex < 16)
		{
			m_world->earse();
			m_world->drawCircle(static_cast<int>(m_worldPoints[m_stepIndex][0]+50.5), 50-static_cast<int>(m_worldPoints[m_stepIndex][1]+0.5), 3, 255, 0, 0);
			m_world->drawCircle(static_cast<int>(m_worldPoints[m_stepIndex][0]+50.5), 50-static_cast<int>(m_worldPoints[m_stepIndex][1]+0.5), 2, 255, 0, 0);
			sprintf(buf, "%c: (%.1lf, %.1lf) -> ", 'A'+m_stepIndex, m_worldPoints[m_stepIndex][0], m_worldPoints[m_stepIndex][1]);
			str += buf;
		}
		else
		{
			m_stepIndex = 16;
			str += tr("calibrating...\n");
			calibrate(m_cameraPoints, m_worldPoints, m_rotate, m_translation, m_f);
			str += "done!\n";
			m_camera->buffer2background();
		}
		m_pointLbl->setText(str);
	}
}

void window::worldBg(void)
{
	QPixmap bg(100,100);
	QPainter p(&bg);
	QPen pen;
	pen.setColor(QColor::fromRgb(0, 0, 0));
	pen.setWidth(1);
	pen.setStyle(Qt::SolidLine);
	bg.fill();
	p.setPen(pen);
	p.drawEllipse(5, 5, 90, 90);
	p.drawEllipse(20, 20, 60, 60);
	p.drawLine(0, 50, 20, 50);
	p.drawLine(80, 50, 100, 50);
	p.drawLine(50, 80, 50, 95);
	p.drawLine(50, 5, 50, 20);
	p.drawLine(97, 48, 100, 48);
	p.drawLine(71, 71, 82, 82);
	p.drawLine(71, 29, 82, 18);
	p.drawLine(29, 71, 18, 82);
	p.drawLine(29, 29, 18, 18);
	m_world->setBackground(&bg);
}

void window::worldMouseDown(QMouseEvent &event)
{
	
	if (16 == m_stepIndex)
	{
		prt(event.x(), event.y());
		m_worldMouseDown = true;
	}
}

void window::worldMouseUp(QMouseEvent &event)
{
	m_worldMouseDown = false;
}

void window::worldMouseMove(QMouseEvent &event)
{
	if (m_worldMouseDown && 16==m_stepIndex)
	{
		prt(event.x(), event.y());
	}
}

void window::prt(int x, int y)
{
	double cx, cy;
	m_world->earse();
	m_world->drawCircle(x, y, 2, 0, 255, 0);
	m_world->drawCircle(x, y, 4, 0, 255, 0);
	convert(x-50, 50-y, cx, cy);
	m_camera->earse();
	m_camera->drawCircle(static_cast<int>(cx*120+384), static_cast<int>(cy*120+288), 2, 0, 0, 255);
	m_camera->drawCircle(static_cast<int>(cx*120+384), static_cast<int>(cy*120+288), 4, 0, 0, 255);
}

void window::convert(int x, int y, double &cx, double &cy)
{
	double tmpz;
	cx = m_rotate[0][0]*x + m_rotate[0][1]*y + m_translation[0];
	cy = m_rotate[1][0]*x + m_rotate[1][1]*y + m_translation[1];
	tmpz = m_rotate[2][0]*x + m_rotate[2][1]*y + m_translation[2];
	cx = (cx / tmpz) * m_f;
	cy = (cy / tmpz) * m_f;
	printf("convert(%d, %d) -> (%.2lf, %.2lf, %.2lf)\n", x, y, cx, cy, tmpz);
}
